﻿using System;
using System.Collections.Generic;
using System.IO;
using Microsoft.Speech.AudioFormat;
using Microsoft.Speech.Synthesis;

namespace Ozeki.Media
{
    public class MSSpeechPlatformTTS : ITextToSpeech
    {
        private SpeechSynthesizer _speech;
        private SpeechAudioFormatInfo _format;

        public MSSpeechPlatformTTS()
            : this(new AudioFormat())
        {
        }

        public MSSpeechPlatformTTS(AudioFormat format)
        {
            _speech = new SpeechSynthesizer();
            _format = new SpeechAudioFormatInfo(format.SampleRate, (AudioBitsPerSample)format.BitRate, (AudioChannel)format.Channels);
            Format = format;
        }

        #region ITextToSpeech Members

        public string Name
        {
            get { return "MSSpeechPlatform"; }
        }

        public int Rate
        {
            get { return _speech.Rate; }
            set { _speech.Rate = value; }
        }

        public string Voice
        {
            get { return _speech.Voice.Name; }
        }

        public AudioFormat Format { get; private set; }

        /// <summary>
        /// Returns the collection of available Text-To-Speech voice.
        /// </summary>
        public List<VoiceInfo> GetAvailableVoices()
        {
            List<VoiceInfo> list = new List<VoiceInfo>();

            try
            {
                var voices = _speech.GetInstalledVoices();
                foreach (var voice in voices)
                {
                    VoiceInfo info = new VoiceInfo(voice.VoiceInfo.Culture.ToString(), voice.VoiceInfo.Name);
                    list.Add(info);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("List exception: {0}", ex.Message);
            }

            return list;
        }


        /// <summary>
        /// Changes the voice of the Text-To-Speech engine.
        /// </summary>
        /// <param name="voice">The name of the given TTS voice.</param>
        public bool ChangeVoice(string voice)
        {
            try
            {
                _speech.SelectVoice(voice);
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine("ChangeVoice method with 1 parameter had error: " + ex.Message);
            }

            return false;
        }

        /// <summary>
        /// Changes the language and voice of the Text-To-Speech engine.
        /// </summary>
        /// <param name="language"></param>
        /// <returns></returns>
        public bool ChangeLanguage(string language)
        {
            try
            {
                _speech.SelectVoice(language);
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine("ChangeLanguage method with 1 parameter had error: " + ex.Message);
            }

            return false;
        }

        /// <summary>
        /// changes the language and voice of the Text-To-Speech engine to the selected "person's"
        /// </summary>
        /// <param name="language"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool ChangeLanguage(string language, string name)
        {
            try
            {
                var voices = _speech.GetInstalledVoices();
                foreach (var voice in voices)
                {
                    Console.WriteLine(voice.VoiceInfo.Culture.ToString());
                    if (voice.VoiceInfo.Culture.ToString().Equals(language))
                    {
                        _speech.SelectVoice(name);
                        return true;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("ChangeLanguage method with 2 parameters had error: " + ex.Message);
            }
            return false;
        }

        /// <summary>
        /// Writes the given text to the playing stream.
        /// </summary>
        public Stream Speak(string text)
        {
            MemoryStream stream = new MemoryStream();
            _speech.SetOutputToAudioStream(stream, _format);
            _speech.Speak(text);
            stream.Position = 0;
            return stream;
        }

        public void Dispose()
        {
            _speech.Dispose();
        }

        #endregion
    }
}
